#!/usr/bin/env python3
#
# A mobility class for RandomWalk on a graph.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# NOTE: this program implements per-node random walk, rather than
# random walk on an edge.  An agent never goes back the currently
# walking edge.

import random

from dtnsim.mobility.graph.fixed import Fixed

class RandomWalk(Fixed):
    def __init__(self, vel_func=None, *kargs, **kwargs):
        super().__init__(*kargs, **kwargs)
        if vel_func is None:
            vel_func = lambda: 1.0  # 1.0 [m/s] by default
        self.velocity = None
        self.vel_func = vel_func

    def update_velocity(self):
        """Update agent's velocity."""
        self.velocity = self.vel_func()
        self.wait = False

    def select_route(self):
        """Randomly choose the next edge to go; this program assumes that the
        agent is currenly on the end of the edge."""
        edge = self.current_edge
        current_vertex = edge[1]
        neighbors = self.path.graph.neighbors(edge[1])
        next_vertex = random.choice(tuple(neighbors))
        self.move_to_point([current_vertex, next_vertex], 0)

    def move(self, delta):
        """Move the agent for the duration of DELTA."""
        self.update_velocity()

        # advance the agent by SPEED * DELTA
        step = self.velocity * delta
        self.current_offset += step
        self.update_current_cache()

        # if very close to either of corners, choose the next route
        length = self.edge_length(self.current_edge)
        if abs(self.current_offset -
               length) < step or self.current_offset > length:
            self.select_route()



#!/usr/bin/env3 python3
#
# A mobility class for stationary agents on a graph.
# Copyright (c) 2013-2018, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import random

from dtnsim.mobility.fixed import Fixed as _Fixed
from perlcompat import die

class Fixed(_Fixed):
    def __init__(self, path=None, *kargs, **kwargs):
        super().__init__(*kargs, **kwargs)
        if not path:
            die("Underlying path class must be specified.")
        if not path.graph:
            die("Path class doesn't return a valid graph via path.graph.")
        self.path = path
        self.current_edge = None
        self.current_offset = None
        self.wait = True
        # choose a random point on a graph
        self.move_to_point(*self.random_point())

    def vertex_coordinate(self, v):
        """Return the coordinate of the vertex V."""
        return self.path.graph.get_vertex_attribute(v, 'xy')

    def distance_between_vertices(self, u, v):
        """Return the Euclid distance between two vertices U and V."""
        return abs(self.vertex_coordinate(u) - self.vertex_coordinate(v))

    def edge_length(self, edge):
        """Return the legnth of edge EDGE."""
        return self.path.graph.get_edge_weight_by_id(*edge, 0)

    def random_offset(self, edge):
        """Randomly choose an offset between zero and the length of edge EDGE."""
        return random.uniform(0, self.edge_length(edge))

    def random_point(self):
        """Pick a random point, which is defined by the edge and the offset,
        on a graph."""
        # FIXME: must choose an edge with a probability proportional to
        # its length
        edge = self.path.graph.random_edge()
        return edge, self.random_offset(edge)

    def get_coordinate(self, edge, offset):
        """Return the coordinate of the point specified by EDGE and OFFSET."""
        pu = self.vertex_coordinate(edge[0])
        pv = self.vertex_coordinate(edge[1])
        length = self.edge_length(edge)
        if length == 0:
            return pu
        return pu + (pv - pu) * offset / length

    def move_to_point(self, edge, offset=0):
        """Directly jump to the point defined by (EDGE, OFFSET)."""
        self.current_edge = edge
        self.current_offset = offset
        self.update_current_cache()

    def vertex_point(self, v):
        """Return the pair (edge, offset) of the vertex V."""
        g = self.path.graph
        neighbors = sorted(g.neighbors(v))
        u = neighbors.pop(0)
        return [v, u], 0

    def move_to_vertex(self, v):
        """Directly jump to the vertex V."""
        self.move_to_point(self.vertex_point(v))

    def update_current_cache(self):
        """Compute and store the current coordinate for later use."""
        self.current = self.get_coordinate(self.current_edge,
                                           self.current_offset)

    def move(self, delta):
        """Move the agent for the duration of DELTA."""
        pass



#!/usr/bin/env3 python3
#
# A mobility class for visiting vertices sequentially.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import random

from dtnsim.mobility.graph.crwp import CRWP

class Sequential(CRWP):
    def __init__(self, next_goal=2, *kargs, **kwargs):
        super().__init__(*kargs, **kwargs)
        self.next_goal = next_goal
        # reposition to vertex 1
        self.move_to_point(self.vertex_point(1))

    def pick_goal(self):
        """Select the next goal; visit vertices in the ascending order of
        their IDs."""
        edge, offset = self.vertex_point(self.next_goal)
        self.goal_edge = edge
        self.goal_offset = offset
        self.update_goal_cache()
        self.reverse_current_if_necessary()

        nvertices = len(self.path.graph.vertices())
        next_goal = self.next_goal + 1
        if next_goal > nvertices:
            next_goal = 1
        self.next_goal = next_goal



#!/usr/bin/env python3
#
# A class for managing the simulator virtual clock.
# Copyright (c) 2018-2019, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

VERSION = '1.4'

def version():
    return """
This is dtnsim, version {}.
Copyright (c) 2010-2019, Hiroyuki Ohsaki.
All rights reserved.""".format(VERSION)

class Scheduler:
    def __init__(self, time=0, delta=10, max_time=10000):
        self.time = time  # simulation clock [s]
        self.delta = delta  # slot length [s]
        self.max_time = max_time
        self.agents = []
        self.zone_cache = {}

    def __repr__(self):
        name = self.__class__.__name__
        return '{}(time={!r}, delta={!r}, max_time={!r})'.format(
            name, self.time, self.delta, self.max_time)

    def advance(self):
        """Advance the simulator clock by DELTA."""
        self.time += self.delta

    def is_running(self):
        """Check if the simulation is running."""
        return self.time <= self.max_time

    def add_agent(self, agent):
        """Register an agent AGENT to the scheduler."""
        self.agents.append(agent)

    def agent_by_id(self, n):
        """Return an agent whose identifier is N.  Th identifier is an integer
        starting from 1."""
        return self.agents[n - 1]

    def cache_zones(self):
        """Refresh the zone information of all agents (i..e., the location of
        grids (zones) in which every agent exists.  The zone information is
        utilized to speed-up the detection of agent-to-agent encounters."""
        self.zone_cache = {}
        for agent in self.agents:
            agent.cache_zone()



#!/usr/bin/env python3
#
# A mobility class for RWP (Random WayPoint) mobility model.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from dtnsim.mobility.randomwalk import RandomWalk

class RandomWaypoint(RandomWalk):
    def __init__(self, pause_func=None, *kargs, **kwargs):
        super().__init__(*kargs, **kwargs)
        if pause_func is None:
            pause_func = lambda: 0.0  # no pause time by default
        self.pause_func = pause_func
        self.wait = 0
        self.goal = self.goal_coordinate()

    def goal_coordinate(self):
        """Randomly choose the goal on the field."""
        return self.random_coordinate()

    def update_velocity(self):
        """Update agent's velocity using the velocity function."""
        self.velocity = self.vel_func() * (
            self.goal - self.current) / abs(self.goal - self.current)

    def move(self, delta):
        """Move the agent for the duration of DELTA."""
        # sleep until wait time expires
        self.wait = max(self.wait - delta, 0)
        if self.wait > 0:
            return

        self.update_velocity()
        self.current += self.velocity * delta

        # if close enough to the goal, randomly choose another goal
        epsilon = abs(self.velocity) * delta
        if abs(self.goal - self.current) <= epsilon:
            self.goal = self.goal_coordinate()
            self.update_velocity()
            self.wait = self.pause_func()



#!/usr/bin/env python3
#
# A mobility class for stationary agents.
# Copyright (c) 2013-2019, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import math, random
from dtnsim.vector import Vector as V

class Fixed:
    def __init__(self, width=1000, height=1000, current=None, *kargs,
                 **kwargs):
        self.width = width
        self.height = height
        if current is None:
            current = self.random_coordinate()
        self.current = current
        self.wait = True

    def __repr__(self):
        name = self.__class__.__name__
        return '{}(width={!r}, height={!r}, current={!r}, wait={!r})'.format(
            name, self.width, self.height, self.current, self.wait)

    def random_coordinate(self):
        """Pick a random coordinate on the field."""
        return V(random.uniform(0, self.width), random.uniform(0, self.height))

    def angle_between_vectors(self, v1, v2):
        """Return the angle between two vectors V1 and V2."""
        if abs(v1) == 0 or abs(v2) == 0:
            return math.pi / 2
        try:
            return math.acos((v1 * v2) / (abs(v1) * abs(v2)))
        except ValueError:
            return 0

    def move(self, delta):
        """Move the agent for the duration of DELTA."""
        pass



#!/usr/bin/env python3
#
# A mobility class for Levy walk.
# Copyright (c) 2011-2015, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import random
import math

from dtnsim.mobility.rwp import RandomWaypoint
from dtnsim.vector import Vector as V

def pareto(scale, shape):
    """Generate a random variable following the Pareto distribution with
    parameters SCALE and SHAPE.  Note that the mean of the Pareto distribution
    is given by SHAPE * SCALE / (SHAPE - 1)."""
    # https://en.wikipedia.org/wiki/Pareto_distribution#Random_sample_generation
    return scale / random.uniform(0, 1) ** (1 / shape)

class LevyWalk(RandomWaypoint):
    def __init__(self, scale=100, shape=1.5, *kargs, **kwargs):
        # NOTE: must be assigned before calling __init__
        self.scale = scale
        self.shape = shape
        super().__init__(*kargs, **kwargs)

    def goal_coordinate(self):
        """Randomly choose the goal in the field so that the distance from the
        current coordinate follows Pareto distribution."""
        length = pareto(self.scale, self.shape)
        theta = random.uniform(0, 2 * math.pi)
        goal = self.current + length * V(math.cos(theta), math.sin(theta))
        # FIXME: the goal coordinate is simply limited by the field boundaries.
        # A node should *bounce back* with the boundaries.
        x = max(0, min(goal[0], self.width))
        y = max(0, min(goal[1], self.height))
        return V(x, y)



#!/usr/bin/env python3
#
# A mobility class for RWP (Random WayPoint) mobility model on limited area.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import random

from dtnsim.mobility.rwp import RandomWaypoint
from dtnsim.vector import Vector as V

class LimitedRandomWaypoint(RandomWaypoint):
    def __init__(self, xmin=0, ymin=0, xmax=1000, ymax=1000, *kargs, **kwargs):
        super().__init__(*kargs, **kwargs)
        self.xmin = xmin
        self.ymin = ymin
        self.xmax = xmax
        self.ymax = ymax

    def random_coordinate(self):
        """Pick a random coordinate on the field."""
        x = random.uniform(self.xmin, self.xmax)
        y = random.uniform(self.ymin, self.ymax)
        return V(x, y)



#!/usr/bin/env python3
#
# A monitor class for visualizing simulation with cell.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from dtnsim.monitor.null import Null

def float2str(v, fmt='9.3f'):
    """Return string representation of a number V using the format FMT.  All
    white spaces are replaced with double-underscores."""
    astr = ('%' + fmt) % v
    astr = astr.replace(' ', '__')
    return astr

def to_geometry(v):
    """Convert the relative length V to the absolute length.  This code
    assumes both the width and the height of the field is 1,000."""
    return v / 1000

class Cell(Null):
    def open(self):
        """Initialize the color palette."""
        print('palette c_edge   heat10 .2')
        print('palette c_vertex heat20 .9')
        print('palette c_sus_range heat20 .3')
        print('palette c_sus       heat20 .9')
        print('palette c_inf_range heat80 .3')
        print('palette c_inf       heat80 .9')
        print('palette c_wait_sus_range heat20 .3')
        print('palette c_wait_sus       heat20 .9')

    def close(self):
        pass

    def display_path(self, path):
        """Draw all underlying paths on the field."""
        graph = path.graph
        if not graph:
            return
        for v in sorted(graph.vertices()):
            p = graph.get_vertex_attribute(v, 'xy')
            x, y = to_geometry(p[0]), to_geometry(p[1])
            print('define v{} ellipse 2 2 c_vertex {} {}'.format(v, x, y))
            #print('define v{0}t text {0} 14 white {1} {2}'.format(v, x, y))
        for u, v in graph.edges():
            print('define - link v{} v{} 1 c_edge'.format(u, v))
        # NOTE: this code assumes paths will not move indefinitely
        print('fix /./')

    def change_agent_status(self, agent):
        """Update the color of agent if it has already received a message."""
        id_ = agent.id_
        color = 'c_sus'
        if agent.mobility.wait:
            color = 'c_wait_sus'
        if agent.received or agent.receive_queue:
            color = 'c_inf'
        print('color agent{} {}'.format(id_, color))
        print('color agentr{} {}_range'.format(id_, color))

    def display_agents(self):
        """Draw all agents on the field."""
        for agent in self.scheduler.agents:
            id_ = agent.id_
            p = agent.mobility.current
            x, y = to_geometry(p[0]), to_geometry(p[1])
            r = to_geometry(agent.range_)
            print('define agent{} ellipse 4 4 white {} {}'.format(id_, x, y))
            print('define agentr{0} ellipse {1} {1} white {2} {3}'.format(
                id_, r, x, y))
            self.change_agent_status(agent)

    def move_agent(self, agent):
        """Reposition the location of the agent AGENT."""
        id_ = agent.id_
        p = agent.mobility.current
        x, y = to_geometry(p[0]), to_geometry(p[1])
        print('move agent{} {} {}'.format(id_, x, y))
        print('move agentr{} {} {}'.format(id_, x, y))

    def display_status(self):
        """Display the current statistics at the top of the screen."""
        time = float2str(self.scheduler.time, '10.2f')
        tx = float2str(self.tx_total, '10g')
        rx = float2str(self.rx_total, '10g')
        dup = float2str(self.dup_total, '10g')
        uniq_total = float2str(self.uniq_total, '10g')
        delivered_total = float2str(self.delivered_total, '10g')
        uniq_delivered_total = float2str(self.uniq_delivered_total, '10g')
        print(
            'define status_l text Time:{},____TX:{},____RX:{},____DUP:{},____Delivered:{}__/__{},____Arrived:{} 14 white 0.5 0.05'
            .format(time, tx, rx, dup, uniq_delivered_total, uniq_total,
                    delivered_total))

    def update(self):
        print('display')



#!/usr/bin/env python3

# https://docs.python.org/3.6/distributing/index.html
# https://setuptools.readthedocs.io/en/latest/setuptools.html#developer-s-guide

import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='dtnsim',
    version='1.6',
    author='Hiroyuki Ohsaki',
    author_email='ohsaki@lsnl.jp',
    description='DTN (Delay/Disruption Tolerant Networking) simulator with several agent/mobility models',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/h-ohsaki/dtnsim",
    packages=setuptools.find_packages(),
    install_requires=['Pytess', 'perlcompat', 'cellx', 'graph_tools'],
    scripts=['bin/dtnsim'],
    classifiers=[
        'Programming Language :: Python :: 3',
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        'Operating System :: OS Independent',
    ],
)



#!/usr/bin/env3 python3
#
# A class for straight underlying path.
# Copyright (c) 2011-2018, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import graph_tools

from dtnsim.path.none import NONE
from dtnsim.vector import Vector as V

class Line(NONE):
    def __init__(self, size=5, *args, **kwargs):
        self.size = size
        super().__init__(*args, **kwargs)

        # create a linear topology
        g = graph_tools.Graph(directed=False)
        self.graph = g
        g.create_graph('lattice', 1, self.size)

        # save the positions of vertices as Vector object
        for v in range(1, self.size + 1):
            x, y = ((v - 0.5) / self.size * self.width, 0.5 * self.height)
            g.set_vertex_attribute(v, 'xy', V(x, y))

        self.compute_edge_lengths()

    def compute_edge_lengths(self):
        """Pre-compute the lengths of all edges."""
        g = self.graph
        for u, v in g.unique_edges():
            length = abs(
                g.get_vertex_attribute(u, 'xy') -
                g.get_vertex_attribute(v, 'xy'))
            g.set_edge_weight_by_id(u, v, 0, length)
            g.set_edge_weight_by_id(v, u, 0, length)



#!/usr/bin/env python3
#
# A simple implementation of vector class.
# Copyright (c) 2018-2019, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import math

class Vector:
    def __init__(self, x=0, y=0):
        self.x = float(x)
        self.y = float(y)

    def __iter__(self):
        return iter([self.x, self.y])

    def __getitem__(self, pos):
        if pos == 0:
            return self.x
        elif pos == 1:
            return self.y
        else:
            raise IndexError

    def __len__(self):
        return 2

    def __repr__(self):
        name = type(self).__name__
        return f'{name}({self.x:.3f}, {self.y:.3f})'

    def __eq__(self, other):
        return self.x == other.x and self.y == other.y

    def __abs__(self):
        return math.hypot(self.x, self.y)

    def __bool__(self):
        return self.x == 0 and self.y == 0

    def __add__(self, other):
        return Vector(self.x + other.x, self.y + other.y)

    def __sub__(self, other):
        return Vector(self.x - other.x, self.y - other.y)

    def __mul__(self, other):
        if isinstance(other, Vector):
            return self.x * other.x + self.y * other.y
        else:
            return Vector(self.x * other, self.y * other)

    def __rmul__(self, other):
        return self.__mul__(other)

    def __truediv__(self, other):
        return Vector(self.x / other, self.y / other)



#!/usr/bin/env3 python3
#
# A class for Voronoi path.
# Copyright (c) 2011-2019, Hiroyuki Ohsaki.
# All rights reserved.
#

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import graph_tools

from dtnsim.path.line import Line
from dtnsim.vector import Vector as V

class Voronoi(Line):
    def __init__(self, npoints=100, *args, **kwargs):
        self.npoints = npoints
        super().__init__(*args, **kwargs)

        # create a Voronoi topology
        g = graph_tools.Graph(directed=False)
        self.graph = g
        g.create_graph('voronoi', self.npoints, self.width, self.height)

        # save the positions of vertices as Vector object
        for v in g.vertices():
            _, x, y = g.get_vertex_attribute(v, 'pos').split('+')
            x, y = float(x), float(y)
            g.set_vertex_attribute(v, 'xy', V(x, y))

        self.compute_edge_lengths()
